#include "config.h"
#include "version.h"

#include <WiFi.h>
#include <WiFiClientSecure.h>
#include <Adafruit_NeoPixel.h>

#include <Arduino_MQTT_Client.h>
#include <Server_Side_RPC.h>
#include <ThingsBoard.h>

// then these pins will be defined for us
#if defined(ADAFRUIT_FEATHER_ESP32_V2) or defined(ARDUINO_ADAFRUIT_ITSYBITSY_ESP32)
#define PIN_NEOPIXEL 0
#define NEOPIXEL_I2C_POWER 2
#endif

#if defined(PIN_NEOPIXEL)
Adafruit_NeoPixel pixel(1, PIN_NEOPIXEL, NEO_GRB + NEO_KHZ800);
#endif

// Initialize underlying client, used to establish a connection
#if ENCRYPTED
WiFiClientSecure espClient;
#else
WiFiClient espClient;
#endif

// Initalize the Mqtt client instance
Arduino_MQTT_Client mqttClient(espClient);

// Maximum size packets will ever be sent or received by the underlying MQTT client,
// if the size is to small messages might not be sent or received messages will be discarded
constexpr uint16_t MAX_MESSAGE_SEND_SIZE		= 128U;
constexpr uint16_t MAX_MESSAGE_RECEIVE_SIZE		= 128U;
constexpr uint8_t  MAX_RPC_SUBSCRIPTIONS		= 8U;
constexpr uint8_t  MAX_RPC_RESPONSE				= 16U;
constexpr uint8_t  MAX_RPC_REQUEST				= 10U;
constexpr uint64_t REQUEST_TIMEOUT_MICROSECONDS = 5000U * 1000U;

// Maximum amount of attributs we can request or subscribe, has to be set both in the ThingsBoard
// template list and Attribute_Request_Callback template list and should be the same as the amount
// of variables in the passed array. If it is less not all variables will be requested or subscribed
constexpr size_t MAX_ATTRIBUTES = 3U;

// Initialize used apis
Server_Side_RPC<MAX_RPC_SUBSCRIPTIONS, MAX_RPC_RESPONSE> server_rpc;
const std::array<IAPI_Implementation*, 1U>				 apis
	= { &server_rpc /*, &client_rpc , &shared_update */ };

// Initialize ThingsBoard instance with the maximum needed buffer size
ThingsBoard tb(
	mqttClient, MAX_MESSAGE_RECEIVE_SIZE, MAX_MESSAGE_SEND_SIZE, Default_Max_Stack_Size, apis);

// Statuses for subscribing to shared attributes
bool subscribed = false;

// Send once
bool once = true;

void setup()
{
// Initalize serial connection for debugging
#if SERIAL_DEBUG
	Serial.begin(SERIAL_DEBUG_BAUD);
	delay(200);
#endif

	// Set VMC, LIGHT, HEATER & AC pin mode as OUTPUT
	pinMode(VMC_PIN, OUTPUT);
	pinMode(LIGHT_PIN, OUTPUT);
	pinMode(HEATER_PIN, OUTPUT);
	pinMode(AC_PIN, OUTPUT);

	// Set initial value LOW for VMC, LIGHT, HEATER & AC pin
	digitalWrite(VMC_PIN, LOW);
	digitalWrite(LIGHT_PIN, LOW);
	digitalWrite(HEATER_PIN, LOW);
	digitalWrite(AC_PIN, LOW);

	// Init Wifi connexion
}

void loop()
{
	tb.loop();
}